#ifndef __ESP32_VS1053_Stream__
#define __ESP32_VS1053_Stream__

#include <Arduino.h>
#include <HTTPClient.h>
#include <VS1053.h>  /* https://github.com/baldram/ESP_VS1053_Library */

#define VS1053_INITIALVOLUME          95
#define VS1053_MAXVOLUME              100
#define VS1053_ICY_METADATA           true
#define VS1053_MAX_PLAYLIST_READ      512
#define VS1053_MAX_URL_LENGTH         512

#define CONNECT_TIMEOUT_MS            250
#define CONNECT_TIMEOUT_MS_SSL        2500

const size_t VS1053_PACKETSIZE = 32;

extern void audio_showstation(const char*) __attribute__((weak));
extern void audio_eof_stream(const char*) __attribute__((weak));
extern void audio_showstreamtitle(const char*) __attribute__((weak));

class ESP32_VS1053_Stream {

    public:
        ESP32_VS1053_Stream();
        ~ESP32_VS1053_Stream();

        bool startDecoder(const uint8_t CS, const uint8_t DCS, const uint8_t DREQ);
        bool isChipConnected();

        bool connecttohost(const char* url);
        bool connecttohost(const char* url, const size_t offset);
        bool connecttohost(const char* url, const char* user, const char* pwd);
        bool connecttohost(const char* url, const char* user, const char* pwd, const size_t offset);

        void loop();
        bool isRunning();
        void stopSong();
        uint8_t getVolume();
        void setVolume(const uint8_t vol); /* 0-100 */
        void setTone(uint8_t *rtone);
        /*  Bass/Treble: void setTone(uint8_t *rtone);
            toneha       = <0..15>        // Setting treble gain (0 off, 1.5dB steps)
            tonehf       = <0..15>        // Setting treble frequency lower limit x 1000 Hz
            tonela       = <0..15>        // Setting bass gain (0 = off, 1dB steps)
            tonelf       = <0..15>        // Setting bass frequency lower limit x 10 Hz
            e.g. uint8_t rtone[4]  = {12, 15, 15, 15}; // initialize bass & treble
            See https://www.vlsi.fi/fileadmin/datasheets/vs1053.pdf section 9.6.3 */
        const char* currentCodec();
        const char* lastUrl();
        size_t size();
        size_t position();
        uint32_t bitrate();
};

#endif
